searchData={"items":[{"type":"module","title":"Genesis","doc":"","ref":"Genesis.html"},{"type":"behaviour","title":"Genesis.Aspect","doc":"Provides common behavior and callbacks for aspects.\nAspects are modular pieces of state or behavior that can be attached to objects.","ref":"Genesis.Aspect.html"},{"type":"callback","title":"Genesis.Aspect.all/0","doc":"Returns all aspects of the given type.\nReturns a list of tuples containing the object and the aspect struct.","ref":"Genesis.Aspect.html#c:all/0"},{"type":"callback","title":"Examples - Genesis.Aspect.all/0","doc":"iex> Health.all()\n    [{1, %Health{current: 100}}, {2, %Health{current: 50}}]","ref":"Genesis.Aspect.html#c:all/0-examples"},{"type":"callback","title":"Genesis.Aspect.at_least/2","doc":"Returns all aspects that have the given property with a value greater than or equal to the given minimum.","ref":"Genesis.Aspect.html#c:at_least/2"},{"type":"callback","title":"Examples - Genesis.Aspect.at_least/2","doc":"iex> Health.at_least(:current, 50)\n    [{1, %Health{current: 75}}]","ref":"Genesis.Aspect.html#c:at_least/2-examples"},{"type":"callback","title":"Genesis.Aspect.at_most/2","doc":"Returns all aspects that have the given property with a value less than or equal to the given maximum.","ref":"Genesis.Aspect.html#c:at_most/2"},{"type":"callback","title":"Examples - Genesis.Aspect.at_most/2","doc":"iex> Health.at_most(:current, 50)\n    [{1, %Health{current: 25}}]","ref":"Genesis.Aspect.html#c:at_most/2-examples"},{"type":"callback","title":"Genesis.Aspect.attach/2","doc":"Attaches an aspect to an object.","ref":"Genesis.Aspect.html#c:attach/2"},{"type":"callback","title":"Genesis.Aspect.between/3","doc":"Returns all aspects that have the given property with a value between the given minimum and maximum.","ref":"Genesis.Aspect.html#c:between/3"},{"type":"callback","title":"Examples - Genesis.Aspect.between/3","doc":"iex> Health.between(:current, 50, 100)\n    [{1, %Health{current: 75}}]","ref":"Genesis.Aspect.html#c:between/3-examples"},{"type":"callback","title":"Genesis.Aspect.cast/1","doc":"Casts the given properties into a map of permitted values.\nThis function normalizes input that can be used to create an aspect.","ref":"Genesis.Aspect.html#c:cast/1"},{"type":"callback","title":"Genesis.Aspect.exists?/1","doc":"Returns true if the aspect is attached to the given object.","ref":"Genesis.Aspect.html#c:exists?/1"},{"type":"callback","title":"Genesis.Aspect.get/1","doc":"Retrieves the aspect attached to an object.\nReturns the aspect struct if present or `nil`.","ref":"Genesis.Aspect.html#c:get/1"},{"type":"callback","title":"Examples - Genesis.Aspect.get/1","doc":"iex> Health.get(1)\n    %Health{current: 100}","ref":"Genesis.Aspect.html#c:get/1-examples"},{"type":"callback","title":"Genesis.Aspect.get/2","doc":"Same as `get/1`, but returns a default value if the aspect is not present.","ref":"Genesis.Aspect.html#c:get/2"},{"type":"callback","title":"Genesis.Aspect.handle_event/3","doc":"Handles events dispatched to this aspect via its parent object.\nReceives the event name, the object and a map of arguments for the event.\n\nGiven that the same event is dispatched to all aspects within an object, this\nfunction should return a tuple with `:cont` or `:halt` to either keep processing\nthe event or stop propagating the event to the remaining aspects in the pipeline.","ref":"Genesis.Aspect.html#c:handle_event/3"},{"type":"callback","title":"Genesis.Aspect.init/0","doc":"Initializes the aspect ETS table.\nShould return an atom with the name of the table and a list of events.","ref":"Genesis.Aspect.html#c:init/0"},{"type":"callback","title":"Genesis.Aspect.match/1","doc":"Returns all aspects that match the given properties.","ref":"Genesis.Aspect.html#c:match/1"},{"type":"callback","title":"Examples - Genesis.Aspect.match/1","doc":"iex> Moniker.match(name: \"Tripida\")\n    [{1, %Moniker{name: \"Tripida\"}}]","ref":"Genesis.Aspect.html#c:match/1-examples"},{"type":"callback","title":"Genesis.Aspect.new/1","doc":"Creates a new aspect by casting the given properties.\nThe given properties are passed to the `cast/1` function.","ref":"Genesis.Aspect.html#c:new/1"},{"type":"type","title":"Genesis.Aspect.object/0","doc":"","ref":"Genesis.Aspect.html#t:object/0"},{"type":"macro","title":"Genesis.Aspect.prop/3","doc":"Defines a property for the aspect.","ref":"Genesis.Aspect.html#prop/3"},{"type":"type","title":"Genesis.Aspect.props/0","doc":"","ref":"Genesis.Aspect.html#t:props/0"},{"type":"callback","title":"Genesis.Aspect.remove/1","doc":"Removes an aspect from an object.\nReturns `:noop`  if the aspect is not present.","ref":"Genesis.Aspect.html#c:remove/1"},{"type":"callback","title":"Genesis.Aspect.update/2","doc":"Updates an aspect attached to an object.\nWill return `:noop` if the aspect is not present.","ref":"Genesis.Aspect.html#c:update/2"},{"type":"module","title":"Genesis.Query","doc":"Provides helper functions to query objects in the registry.","ref":"Genesis.Query.html"},{"type":"function","title":"Genesis.Query.all_of/1","doc":"Returns a list of objects that have all the aspects specified in the list.","ref":"Genesis.Query.html#all_of/1"},{"type":"function","title":"Examples - Genesis.Query.all_of/1","doc":"iex> Query.all_of([Aspect1, Aspect2])\n    [{1, [Aspect1, Aspect2]}, {2, [Aspect1, Aspect2]}]","ref":"Genesis.Query.html#all_of/1-examples"},{"type":"function","title":"Genesis.Query.any_of/1","doc":"Returns a list of objects that have at least one of the aspects specified in the list.","ref":"Genesis.Query.html#any_of/1"},{"type":"function","title":"Examples - Genesis.Query.any_of/1","doc":"iex> Query.any_of([Aspect1, Aspect2])\n    [{1, [Aspect1]}, {2, [Aspect2]}, {3, [Aspect1, Aspect2]}]","ref":"Genesis.Query.html#any_of/1-examples"},{"type":"function","title":"Genesis.Query.none_of/1","doc":"Returns a list of objects that do not have any of the aspects specified in the list.","ref":"Genesis.Query.html#none_of/1"},{"type":"function","title":"Examples - Genesis.Query.none_of/1","doc":"iex> Query.none_of([Aspect1, Aspect2])\n    [{3, [Aspect3, Aspect4]}, {4, [Aspect4, Aspect5]}]","ref":"Genesis.Query.html#none_of/1-examples"},{"type":"function","title":"Genesis.Query.query/1","doc":"Returns a list of objects that match the specified criteria.\nThe function allows grouping the behavior of `all_of/1`, `any_of/1`, and `none_of/1`.","ref":"Genesis.Query.html#query/1"},{"type":"function","title":"Examples - Genesis.Query.query/1","doc":"iex> Query.query(all: [Aspect1], any: [Aspect2], none: [Aspect3])\n    [{1, [Aspect1, Aspect2]}, {2, [Aspect1]}]","ref":"Genesis.Query.html#query/1-examples"},{"type":"module","title":"Genesis.World","doc":"The World is a GenServer that acts as a registry and manages the lifecycle of objects and aspects.\nIt is responsible for creating, cloning, and destroying objects, as well as registering aspects and prefabs.\nIt also manages the event routing for objects and aspects, ensuring that events are dispatched to the correct handlers.","ref":"Genesis.World.html"},{"type":"function","title":"Genesis.World.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Genesis.World.html#child_spec/1"},{"type":"function","title":"Genesis.World.clone/1","doc":"Clones an object with all the aspects as the original object.","ref":"Genesis.World.html#clone/1"},{"type":"function","title":"Genesis.World.create/1","doc":"Creates a new object from a prefab.\nThe prefab must be registered in the World before it can be used.","ref":"Genesis.World.html#create/1"},{"type":"function","title":"Genesis.World.destroy/1","doc":"Destroys an object from the world.","ref":"Genesis.World.html#destroy/1"},{"type":"function","title":"Genesis.World.fetch/1","doc":"Fetches the aspects of an object.","ref":"Genesis.World.html#fetch/1"},{"type":"function","title":"Genesis.World.list_aspects/0","doc":"List all aspects registered in the world.","ref":"Genesis.World.html#list_aspects/0"},{"type":"function","title":"Genesis.World.list_objects/0","doc":"List all objects spawned in the world.","ref":"Genesis.World.html#list_objects/0"},{"type":"function","title":"Genesis.World.new/0","doc":"Creates a new unique object ID.","ref":"Genesis.World.html#new/0"},{"type":"function","title":"Genesis.World.register_aspect/1","doc":"Registers an aspect module.","ref":"Genesis.World.html#register_aspect/1"},{"type":"function","title":"Genesis.World.register_prefab/1","doc":"Registers a new prefab.","ref":"Genesis.World.html#register_prefab/1"},{"type":"function","title":"Genesis.World.send/2","doc":"Sends a message to an object.","ref":"Genesis.World.html#send/2"},{"type":"function","title":"Genesis.World.start_link/1","doc":"Starts the World process.","ref":"Genesis.World.html#start_link/1"},{"type":"extras","title":"Genesis","doc":"# Genesis\n\nGenesis is an ECS (Entity Component System) library for Elixir focused on easy of use and ergonomics. This library is heavily inspired by the architecture used in the game [Caves of Qud](https://www.cavesofqud.com/) and [ADOM (Ancient Domains of Mystery)](https://www.adom.de/).\n\n> [!TIP]\n> If you don't yet know what an Entity Component System is, the [ECS FAQ repository](https://github.com/SanderMertens/ecs-faq) from the creator of [Flecs](https://www.flecs.dev/) (a very popular ECS library) is a very good resource to get a good overview.\n\nIn the case of Genesis, the ECS terminology is used quite loosely as we don't make any assumptions about your game loop looks like. Genesis was created to take advantage Elixir strenghts to create an event-driven architecture that is very usefull for building turn-based games. In fact, the whole idea for this library is based on these talks from Thomas Biskup and Briand Bucklew:\n\n- [AI in Qud and Sproggiwood](https://www.youtube.com/watch?v=4uxN5GqXcaA)\n- [Data-Driven Engines of Qud and Sproggiwood](https://www.youtube.com/watch?v=U03XXzcThGU)\n- [There be dragons: Entity Component Systems for Roguelikes](https://www.youtube.com/watch?v=fGLJC5UY2o4)","ref":"README.html"},{"type":"extras","title":"Installation - Genesis","doc":"```elixir\ndef deps do\n  [\n    {:genesis, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"README.html#installation"},{"type":"extras","title":"Special Thanks ❤️ - Genesis","doc":"A big thanks to both Brian Bucklew and Thomas Biskup for the inspiring talks. The Caves of Qud modding guides in particular was a great resource to see what this architecture would be capable of.\n\nSpecial thanks to other ECS libraries that influenced the development of Genesis:\n\n- [Geotic](https://github.com/ddmills/geotic) - A friendly JavaScript ECS library that is also based on the the Briand Bucklew and Thomas Biskup talks. Finding this library in the midst of traditional ECS implementations was a breath of fresh.\n- [ECSx](https://github.com/ecsx-framework/ECSx) - Another excellent ECS library with a more traditional approach. ECSx paved the way as one of the very first Elixir implementations. It greatly impacted some of Genesis's design choices.","ref":"README.html#special-thanks"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}